#include "i2cmaster.h"
#include "pcf8583.h"

#define RTCLK  0xA0	// i2c device address of RTC PCF8583 on slot0 -->0xA0
#define BEG_FREE_RAM	0x10
		
unsigned char to_BCD(unsigned char natural_binary)
{
   return(((natural_binary / 10) << 4) | natural_binary % 10);
}

unsigned char to_natural_binary(unsigned char BCD)
{
   return(((BCD >> 4) * 10) + (BCD & 0x0f));
}

void pcf8583_control(unsigned char control_reg)
{  
  i2c_start_wait(RTCLK + I2C_WRITE); 
  i2c_write(0x00);
  i2c_write(control_reg);
  i2c_stop();
}

void pcf8583_store_year(unsigned char yr)
{
  i2c_start_wait(RTCLK + I2C_WRITE);
  i2c_write(BEG_FREE_RAM);
  i2c_write('O');				/* write a signature, any pattern is valid */
  i2c_write('K');
  i2c_write(yr);				/* store (year - YEAR_BASE) at addr 0x12 (free RAM) */
  i2c_stop();
}

void pcf8583_write(struct Time *p_t, struct Date *p_d)
{
/* Note that most significant bit of hours is 12/24 hour format
** 4 year is in bits 7 and 6 of 0x05.  Lower six bits are day in BCD
** location 0x06.  Weeks day is in bits 7, 6, 5 and month in lower
** five bits.
*/  
  i2c_start_wait(RTCLK + I2C_WRITE); 
  i2c_write(0x01); 				/* address of first write */
  i2c_write(0x00); 				/* hundreths of a second */
  i2c_write(to_BCD(p_t->se));
  i2c_write(to_BCD(p_t->mi));
  i2c_write(to_BCD(p_t->hr));			/* 24 hour format - 0x80 for 12 hour time */
  i2c_write((((YEAR_BASE + p_d->yr) & 3) << 6)|to_BCD(p_d->da));	/* YY TT UUUU */
  i2c_write((p_d->weekday << 5)|to_BCD(p_d->mo));/* DDD T UUUU */
  i2c_stop();

}

void pcf8583_read(struct Time *p_t, struct Date *p_d)
{ 
  unsigned char d, y;
  i2c_start_wait(RTCLK + I2C_WRITE); 
  i2c_write(0x02); 				/* begin with seconds */
  i2c_rep_start(RTCLK + I2C_READ);
  p_t->se = to_natural_binary(i2c_readAck() & 0x7f);
  p_t->mi = to_natural_binary(i2c_readAck());
  p_t->hr = to_natural_binary(i2c_readAck() & 0x3f);
  d = i2c_readAck();
  y = d >> 6;    				/* year is in two most sig bits */
  p_d->da = to_natural_binary(d & 0x3f);	/* day in lower six bits */
  d = i2c_readNak();
  p_d->weekday = d >> 5;			/* day of the week */
  p_d->mo = to_natural_binary(d & 0x1f);	/* read month */
  i2c_stop();
  if (((YEAR_BASE + p_d->yr) & 3) != y) {	/* year has changed */
      p_d->yr ++;   	 			
      pcf8583_store_year(p_d->yr);		/* store (year - YEAR_BASE) at address 0x12 (free RAM) */
  }
}

unsigned char pcf8583_check(void)
{
  unsigned char ret = 0;
  i2c_start_wait(RTCLK + I2C_WRITE); 		/* check first two bytes of free ram to know */
  i2c_write(BEG_FREE_RAM); 			/* if date previously stored is still present */
  i2c_rep_start(RTCLK + I2C_READ);
  if ((i2c_readAck() == 'O') &&			/* verify signature */
      (i2c_readAck() == 'K')) 
  {
     ret = i2c_readNak();			/* return (year - YEAR_BASE) */
  }
  i2c_stop();       
  return ret;
}

void pcf8583_setclk(struct Time *p_t, struct Date *p_d)
{
  pcf8583_control(0x80);	// initialize RTC, 32.768 kHz, no alarm, stop counters & reset dividers
  pcf8583_write(p_t, p_d);	// set time & date
  pcf8583_store_year(p_d->yr);	// store (year - YEAR_BASE) at addr 0x12 (free RAM) of pcf8583
  pcf8583_control(0x00);	// initialize RTC, 32.768 kHz, no alarm, start counters
}
